class CueCloudRequest

  include HTTParty

  #uncomment the following line to debug the entire http party and have fun !
  #debug_output $stdout

  attr_reader :auth_key, :api_key, :api_pass, :base_url, :body, :logger

  # base_uri CueCloudApi::DEFAULT_BASE_URL + CueCloudApi::API_VERSION

  def initialize(api_key, api_pass,base_url=nil)
    @api_key = api_key.to_s
    @api_pass = api_pass.to_s
    @auth_key = {username: api_key, password: api_pass}
    if base_url
      @base_url = base_url
    else
      @base_url = CueCloudApi::DEFAULT_BASE_URL + CueCloudApi::API_VERSION
    end
  end

  def build_request(resource, method, data=nil)

    url = build_url(resource)
    body = prepare_body(method, data)
    nonce = generate_nonce

    message = nonce + url + body

    headers = {
        'Access-Key' => api_key,
        'Access-Signature' => signature(message),
        'Access-Nonce' => nonce,
        'Content-Type' => 'application/json',
    }

    request_call(method, url, auth_key, headers, data)

  end

  private

  def build_url(resource)
    base_url + "/" + resource
  end

  def prepare_body(method, data)
    if data && method =~ /post/i
      data.to_json
    else
      ""
    end
  end

  def request_call(method, url, auth_key, headers, data)
    case method
      when /get/i
        body = self.class.get(url, basic_auth: auth_key, headers: headers).response.body
        return JSON.parse(body)
      when /post/i
        body = self.class.post(url, basic_auth: auth_key, headers: headers, body: data.to_json).response.body
        return JSON.parse(body)
      when /put/i
      when /delete/i
      else
        raise CueCloudException.new("Invalid method:#{method}")
    end
  end

  def signature(message)
    OpenSSL::HMAC.hexdigest(OpenSSL::Digest.new('sha256'), api_pass, message)
  end

  def generate_nonce
    (Time.now.to_f * 1e6).to_i.to_s
  end

end

